@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2000-2002  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2003-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2003-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -50,7 +50,7 @@ static inline uint16_t get_manufacturer(void)
 	return (manufacturer == DEFAULT_COMPID ? parser.defcompid : manufacturer);
 }
 
-#define EVENT_NUM 47
+#define EVENT_NUM 55
 static char *event_str[EVENT_NUM + 1] = {
 	"Unknown",
 	"Inquiry Complete",
@@ -100,6 +100,14 @@ static char *event_str[EVENT_NUM + 1] = {
 	"Synchronous Connect Changed",
 	"Sniff Subrate",
 	"Extended Inquiry Result",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Link Supervision Timeout Change",
 };
 
 #define CMD_LINKCTL_NUM 41
@@ -522,7 +530,7 @@ static inline void bdaddr_command_dump(int level, struct frame *frm)
 	frm->len -= sizeof(bdaddr_t);
 
 	p_indent(level, frm);
-	ba2str(bdaddr, addr);
+	p_ba2str(bdaddr, addr);
 	printf("bdaddr %s\n", addr);
 
 	raw_dump(level, frm);
@@ -555,7 +563,7 @@ static inline void create_conn_dump(int level, struct frame *frm)
 	char addr[18], *str;
 
 	p_indent(level, frm);
-	ba2str(&cp->bdaddr, addr);
+	p_ba2str(&cp->bdaddr, addr);
 	printf("bdaddr %s ptype 0x%4.4x rswitch 0x%2.2x clkoffset 0x%4.4x%s\n",
 		addr, ptype, cp->role_switch,
 		clkoffset & 0x7fff, clkoffset & 0x8000 ? " (valid)" : "");
@@ -602,7 +610,7 @@ static inline void accept_conn_req_dump(int level, struct frame *frm)
 	char addr[18];
 
 	p_indent(level, frm);
-	ba2str(&cp->bdaddr, addr);
+	p_ba2str(&cp->bdaddr, addr);
 	printf("bdaddr %s role 0x%2.2x\n", addr, cp->role);
 
 	p_indent(level, frm);
@@ -615,7 +623,7 @@ static inline void reject_conn_req_dump(int level, struct frame *frm)
 	char addr[18];
 
 	p_indent(level, frm);
-	ba2str(&cp->bdaddr, addr);
+	p_ba2str(&cp->bdaddr, addr);
 	printf("bdaddr %s reason 0x%2.2x\n", addr, cp->reason);
 
 	p_indent(level, frm);
@@ -628,7 +636,7 @@ static inline void pin_code_reply_dump(int level, struct frame *frm)
 	char addr[18], pin[17];
 
 	p_indent(level, frm);
-	ba2str(&cp->bdaddr, addr);
+	p_ba2str(&cp->bdaddr, addr);
 	memset(pin, 0, sizeof(pin));
 	if (parser.flags & DUMP_NOVENDOR)
 		memset(pin, '*', cp->pin_len);
@@ -644,7 +652,7 @@ static inline void link_key_reply_dump(int level, struct frame *frm)
 	int i;
 
 	p_indent(level, frm);
-	ba2str(&cp->bdaddr, addr);
+	p_ba2str(&cp->bdaddr, addr);
 	printf("bdaddr %s key ", addr);
 	for (i = 0; i < 16; i++)
 		if (parser.flags & DUMP_NOVENDOR)
@@ -660,7 +668,7 @@ static inline void pin_code_neg_reply_dump(int level, struct frame *frm)
 	char addr[18];
 
 	p_indent(level, frm);
-	ba2str(bdaddr, addr);
+	p_ba2str(bdaddr, addr);
 	printf("bdaddr %s\n", addr);
 }
 
@@ -679,7 +687,7 @@ static inline void remote_name_req_dump(int level, struct frame *frm)
 	char addr[18];
 
 	p_indent(level, frm);
-	ba2str(&cp->bdaddr, addr);
+	p_ba2str(&cp->bdaddr, addr);
 	printf("bdaddr %s mode %d clkoffset 0x%4.4x%s\n",
 		addr, cp->pscan_rep_mode,
 		clkoffset & 0x7fff, clkoffset & 0x8000 ? " (valid)" : "");
@@ -745,8 +753,7 @@ static inline void sniff_subrate_dump(int level, struct frame *frm)
 	printf("handle %d\n", btohs(cp->handle));
 
 	p_indent(level, frm);
-	printf("max latency remote %d local %d\n",
-		btohs(cp->max_remote_latency), btohs(cp->max_local_latency));
+	printf("max latency %d\n", btohs(cp->max_latency));
 
 	p_indent(level, frm);
 	printf("min timeout remote %d local %d\n",
@@ -795,7 +802,7 @@ static inline void set_event_flt_dump(int level, struct frame *frm)
 				dev_mask[2], dev_mask[1], dev_mask[0]);
 			break;
 		case INQ_RESULT_RETURN_BDADDR:
-			ba2str((bdaddr_t *) cp->condition, addr);
+			p_ba2str((bdaddr_t *) cp->condition, addr);
 			printf(" with bdaddr %s\n", addr);
 			break;
 		default:
@@ -832,7 +839,7 @@ static inline void request_stored_link_key_dump(int level, struct frame *frm)
 	char addr[18];
 
 	p_indent(level, frm);
-	ba2str(&cp->bdaddr, addr);
+	p_ba2str(&cp->bdaddr, addr);
 	printf("bdaddr %s all %d\n", addr, cp->read_all);
 }
 
@@ -844,7 +851,7 @@ static inline void return_link_keys_dump(int level, struct frame *frm)
 	int i, n;
 
 	for (n = 0; n < num; n++) {
-		ba2str(frm->ptr, addr);
+		p_ba2str(frm->ptr, addr);
 		memcpy(key, frm->ptr + 6, 16);
 
 		p_indent(level, frm);
@@ -963,7 +970,7 @@ static inline void write_link_supervision_timeout_dump(int level, struct frame *
 
 	p_indent(level, frm);
 	printf("handle %d timeout %d\n",
-				btohs(cp->handle), btohs(cp->link_sup_to));
+				btohs(cp->handle), btohs(cp->timeout));
 }
 
 static inline void write_ext_inquiry_response_dump(int level, struct frame *frm)
@@ -1007,6 +1014,31 @@ static inline void request_clock_dump(int level, struct frame *frm)
 					cp->which_clock ? "piconet" : "local");
 }
 
+static inline void host_buffer_size_dump(int level, struct frame *frm)
+{
+	host_buffer_size_cp *cp = frm->ptr;
+
+	p_indent(level, frm);
+	printf("ACL MTU %d:%d SCO MTU %d:%d\n",
+				btohs(cp->acl_mtu), btohs(cp->acl_max_pkt),
+				cp->sco_mtu, btohs(cp->sco_max_pkt));
+}
+
+static inline void num_comp_pkts_dump(int level, struct frame *frm)
+{
+	uint8_t num = get_u8(frm);
+	uint16_t handle, packets;
+	int i;
+
+	for (i = 0; i < num; i++) {
+		handle = btohs(htons(get_u16(frm)));
+		packets = btohs(htons(get_u16(frm)));
+
+		p_indent(level, frm);
+		printf("handle %d packets %d\n", handle, packets);
+	}
+}
+
 static inline void command_dump(int level, struct frame *frm)
 {
 	hci_command_hdr *hdr = frm->ptr;
@@ -1169,6 +1201,7 @@ static inline void command_dump(int level, struct frame *frm)
 			return;
 		case OCF_WRITE_SCAN_ENABLE:
 		case OCF_WRITE_AUTH_ENABLE:
+		case OCF_SET_CONTROLLER_TO_HOST_FC:
 			write_scan_enable_dump(level + 1, frm);
 			return;
 		case OCF_WRITE_CONN_ACCEPT_TIMEOUT:
@@ -1190,6 +1223,12 @@ static inline void command_dump(int level, struct frame *frm)
 		case OCF_READ_TRANSMIT_POWER_LEVEL:
 			request_transmit_power_level_dump(level + 1, frm);
 			return;
+		case OCF_HOST_BUFFER_SIZE:
+			host_buffer_size_dump(level + 1, frm);
+			return;
+		case OCF_HOST_NUM_COMP_PKTS:
+			num_comp_pkts_dump(level + 1, frm);
+			return;
 		case OCF_FLUSH:
 		case OCF_READ_LINK_SUPERVISION_TIMEOUT:
 			generic_command_dump(level + 1, frm);
@@ -1263,7 +1302,7 @@ static inline void bdaddr_response_dump(int level, struct frame *frm)
 	frm->len -= sizeof(bdaddr_t);
 
 	p_indent(level, frm);
-	ba2str(bdaddr, addr);
+	p_ba2str(bdaddr, addr);
 	printf("status 0x%2.2x bdaddr %s\n", status, addr);
 
 	if (status > 0) {
@@ -1488,7 +1527,7 @@ static inline void read_link_supervision_timeout_dump(int level, struct frame *f
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d timeout %d\n",
-			rp->status, btohs(rp->handle), btohs(rp->link_sup_to));
+			rp->status, btohs(rp->handle), btohs(rp->timeout));
 
 	if (rp->status > 0) {
 		p_indent(level, frm);
@@ -1735,6 +1774,7 @@ static inline void cmd_complete_dump(int level, struct frame *frm)
 		case OCF_INQUIRY_CANCEL:
 		case OCF_PERIODIC_INQUIRY:
 		case OCF_EXIT_PERIODIC_INQUIRY:
+		case OCF_READ_REMOTE_EXT_FEATURES:
 			status_response_dump(level, frm);
 			return;
 		case OCF_CREATE_CONN_CANCEL:
@@ -1837,6 +1877,8 @@ static inline void cmd_complete_dump(int level, struct frame *frm)
 		case OCF_WRITE_AFH_MODE:
 		case OCF_SET_AFH_CLASSIFICATION:
 		case OCF_WRITE_EXT_INQUIRY_RESPONSE:
+		case OCF_SET_CONTROLLER_TO_HOST_FC:
+		case OCF_HOST_BUFFER_SIZE:
 			status_response_dump(level, frm);
 			return;
 		}
@@ -1927,7 +1969,7 @@ static inline void inq_result_dump(int level, struct frame *frm)
 	for (i = 0; i < num; i++) {
 		inquiry_info *info = frm->ptr;
 
-		ba2str(&info->bdaddr, addr);
+		p_ba2str(&info->bdaddr, addr);
 
 		p_indent(level, frm);
 		printf("bdaddr %s mode %d clkoffset 0x%4.4x class 0x%2.2x%2.2x%2.2x\n",
@@ -1944,7 +1986,7 @@ static inline void conn_complete_dump(int level, struct frame *frm)
 	evt_conn_complete *evt = frm->ptr;
 	char addr[18];
 
-	ba2str(&evt->bdaddr, addr);
+	p_ba2str(&evt->bdaddr, addr);
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d bdaddr %s type %s encrypt 0x%2.2x\n",
@@ -1962,7 +2004,7 @@ static inline void conn_request_dump(int level, struct frame *frm)
 	evt_conn_request *evt = frm->ptr;
 	char addr[18];
 
-	ba2str(&evt->bdaddr, addr);
+	p_ba2str(&evt->bdaddr, addr);
 
 	p_indent(level, frm);
 	printf("bdaddr %s class 0x%2.2x%2.2x%2.2x type %s\n",
@@ -1993,7 +2035,7 @@ static inline void remote_name_req_complete_dump(int level, struct frame *frm)
 	char addr[18], name[249];
 	int i;
 
-	ba2str(&evt->bdaddr, addr);
+	p_ba2str(&evt->bdaddr, addr);
 
 	memset(name, 0, sizeof(name));
 	for (i = 0; i < 248 && evt->name[i]; i++)
@@ -2112,7 +2154,7 @@ static inline void role_change_dump(int level, struct frame *frm)
 	char addr[18];
 
 	p_indent(level, frm);
-	ba2str(&evt->bdaddr, addr);
+	p_ba2str(&evt->bdaddr, addr);
 	printf("status 0x%2.2x bdaddr %s role 0x%2.2x\n",
 						evt->status, addr, evt->role);
 
@@ -2125,21 +2167,6 @@ static inline void role_change_dump(int level, struct frame *frm)
 	}
 }
 
-static inline void num_comp_pkts_dump(int level, struct frame *frm)
-{
-	uint8_t num = get_u8(frm);
-	uint16_t handle, packets;
-	int i;
-
-	for (i = 0; i < num; i++) {
-		handle = btohs(htons(get_u16(frm)));
-		packets = btohs(htons(get_u16(frm)));
-
-		p_indent(level, frm);
-		printf("handle %d packets %d\n", handle, packets);
-	}
-}
-
 static inline void mode_change_dump(int level, struct frame *frm)
 {
 	evt_mode_change *evt = frm->ptr;
@@ -2163,7 +2190,7 @@ static inline void pin_code_req_dump(int level, struct frame *frm)
 	char addr[18];
 
 	p_indent(level, frm);
-	ba2str(&evt->bdaddr, addr);
+	p_ba2str(&evt->bdaddr, addr);
 	printf("bdaddr %s\n", addr);
 }
 
@@ -2174,7 +2201,7 @@ static inline void link_key_notify_dump(int level, struct frame *frm)
 	int i;
 
 	p_indent(level, frm);
-	ba2str(&evt->bdaddr, addr);
+	p_ba2str(&evt->bdaddr, addr);
 	printf("bdaddr %s key ", addr);
 	for (i = 0; i < 16; i++)
 		if (parser.flags & DUMP_NOVENDOR)
@@ -2243,7 +2270,7 @@ static inline void pscan_rep_mode_change_dump(int level, struct frame *frm)
 	char addr[18];
 
 	p_indent(level, frm);
-	ba2str(&evt->bdaddr, addr);
+	p_ba2str(&evt->bdaddr, addr);
 	printf("bdaddr %s mode %d\n", addr, evt->pscan_rep_mode);
 }
 
@@ -2288,7 +2315,7 @@ static inline void inq_result_with_rssi_dump(int level, struct frame *frm)
 
 			p_indent(level, frm);
 
-			ba2str(&info->bdaddr, addr);
+			p_ba2str(&info->bdaddr, addr);
 			printf("bdaddr %s mode %d clkoffset 0x%4.4x class 0x%2.2x%2.2x%2.2x rssi %d\n",
 				addr, info->pscan_rep_mode, btohs(info->clock_offset),
 				info->dev_class[2], info->dev_class[1], info->dev_class[0], info->rssi);
@@ -2302,7 +2329,7 @@ static inline void inq_result_with_rssi_dump(int level, struct frame *frm)
 
 			p_indent(level, frm);
 
-			ba2str(&info->bdaddr, addr);
+			p_ba2str(&info->bdaddr, addr);
 			printf("bdaddr %s mode %d clkoffset 0x%4.4x class 0x%2.2x%2.2x%2.2x rssi %d\n",
 				addr, info->pscan_rep_mode, btohs(info->clock_offset),
 				info->dev_class[2], info->dev_class[1], info->dev_class[0], info->rssi);
@@ -2340,7 +2367,7 @@ static inline void sync_conn_complete_dump(int level, struct frame *frm)
 	evt_sync_conn_complete *evt = frm->ptr;
 	char addr[18];
 
-	ba2str(&evt->bdaddr, addr);
+	p_ba2str(&evt->bdaddr, addr);
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d bdaddr %s type %s\n",
@@ -2381,9 +2408,9 @@ static inline void sniff_subrate_event_dump(int level, struct frame *frm)
 		printf("Error: %s\n", status2str(evt->status));
 	} else {
 		p_indent(level, frm);
-		printf("max latency remote %d local %d\n",
-					btohs(evt->max_remote_latency),
-					btohs(evt->max_local_latency));
+		printf("max latency transmit %d receive %d\n",
+					btohs(evt->max_tx_latency),
+					btohs(evt->max_rx_latency));
 
 		p_indent(level, frm);
 		printf("min timeout remote %d local %d\n",
@@ -2401,7 +2428,7 @@ static inline void extended_inq_result_dump(int level, struct frame *frm)
 	for (i = 0; i < num; i++) {
 		extended_inquiry_info *info = frm->ptr;
 
-		ba2str(&info->bdaddr, addr);
+		p_ba2str(&info->bdaddr, addr);
 
 		p_indent(level, frm);
 		printf("bdaddr %s mode %d clkoffset 0x%4.4x class 0x%2.2x%2.2x%2.2x rssi %d\n",
@@ -2415,6 +2442,15 @@ static inline void extended_inq_result_dump(int level, struct frame *frm)
 	}
 }
 
+static inline void link_supervision_timeout_change_dump(int level, struct frame *frm)
+{
+	evt_link_supervision_timeout_change *evt = frm->ptr;
+
+	p_indent(level, frm);
+	printf("handle %d timeout %d\n",
+				btohs(evt->handle), btohs(evt->timeout));
+}
+
 static inline void event_dump(int level, struct frame *frm)
 {
 	hci_event_hdr *hdr = frm->ptr;
@@ -2431,13 +2467,24 @@ static inline void event_dump(int level, struct frame *frm)
 		p_indent(level, frm);
 		printf("HCI Event: Testing (0x%2.2x) plen %d\n", hdr->evt, hdr->plen);
 	} else if (hdr->evt == EVT_VENDOR) {
+		uint16_t manufacturer;
+
 		if (parser.flags & DUMP_NOVENDOR)
 			return;
 
 		p_indent(level, frm);
 		printf("HCI Event: Vendor (0x%2.2x) plen %d\n", hdr->evt, hdr->plen);
 
-		if (get_manufacturer() == 10) {
+		manufacturer = get_manufacturer();
+
+		switch (manufacturer) {
+		case 0:
+		case 48:
+			frm->ptr += HCI_EVENT_HDR_SIZE;
+			frm->len -= HCI_EVENT_HDR_SIZE;
+			ericsson_dump(level + 1, frm);
+			return;
+		case 10:
 			frm->ptr += HCI_EVENT_HDR_SIZE;
 			frm->len -= HCI_EVENT_HDR_SIZE;
 			csr_dump(level + 1, frm);
@@ -2578,6 +2625,9 @@ static inline void event_dump(int level, struct frame *frm)
 	case EVT_EXTENDED_INQUIRY_RESULT:
 		extended_inq_result_dump(level + 1, frm);
 		break;
+	case EVT_LINK_SUPERVISION_TIMEOUT_CHANGE:
+		link_supervision_timeout_change_dump(level + 1, frm);
+		break;
 	default:
 		raw_dump(level, frm);
 		break;
@@ -2613,9 +2663,10 @@ static inline void sco_dump(int level, struct frame *frm)
 {
 	hci_sco_hdr *hdr = (void *) frm->ptr;
 	uint16_t handle = btohs(hdr->handle);
+	int len;
 
-	if (frm->audio_fd > 2)
-		write(frm->audio_fd, frm->ptr + HCI_SCO_HDR_SIZE, hdr->dlen);
+	if (frm->audio_fd > fileno(stderr))
+		len = write(frm->audio_fd, frm->ptr + HCI_SCO_HDR_SIZE, hdr->dlen);
 
 	if (!p_filter(FILT_SCO)) {
 		p_indent(level, frm);
