@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2000-2002  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2003-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2003-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -699,8 +699,12 @@ static void l2cap_parse(int level, struct frame *frm)
 					hdr->code, hdr->ident, btohs(hdr->len));
 				raw_dump(level, frm);
 			}
-			frm->ptr += btohs(hdr->len);
-			frm->len -= btohs(hdr->len);
+
+			if (frm->len > btohs(hdr->len)) {
+				frm->len -= btohs(hdr->len);
+				frm->ptr += btohs(hdr->len);
+			} else
+				frm->len = 0;
 		}
 	} else if (cid == 0x2) {
 		/* Connectionless channel */
@@ -794,6 +798,13 @@ static void l2cap_parse(int level, struct frame *frm)
 				raw_dump(level + 1, frm);
 			break;
 
+		case 0x17:
+			if (!p_filter(FILT_AVCTP))
+				avctp_dump(level, frm);
+			else
+				raw_dump(level + 1, frm);
+			break;
+
 		case 0x19:
 			if (!p_filter(FILT_AVDTP))
 				avdtp_dump(level, frm);
@@ -861,18 +872,19 @@ void l2cap_dump(int level, struct frame *frm)
 			return;
 		}
 		memcpy(fr->data, frm->ptr, frm->len);
-		fr->data_len = dlen + L2CAP_HDR_SIZE;
-		fr->len      = frm->len;
-		fr->ptr      = fr->data;
-		fr->dev_id   = frm->dev_id;
-		fr->in       = frm->in;
-		fr->ts       = frm->ts;
-		fr->handle   = frm->handle;
-		fr->cid      = frm->cid;
-		fr->num      = frm->num;
-		fr->dlci     = frm->dlci;
-		fr->channel  = frm->channel;
-		fr->audio_fd = frm->audio_fd;
+		fr->data_len   = dlen + L2CAP_HDR_SIZE;
+		fr->len        = frm->len;
+		fr->ptr        = fr->data;
+		fr->dev_id     = frm->dev_id;
+		fr->in         = frm->in;
+		fr->ts         = frm->ts;
+		fr->handle     = frm->handle;
+		fr->cid        = frm->cid;
+		fr->num        = frm->num;
+		fr->dlci       = frm->dlci;
+		fr->channel    = frm->channel;
+		fr->pppdump_fd = frm->pppdump_fd;
+		fr->audio_fd   = frm->audio_fd;
 	} else {
 		if (!(fr = get_frame(frm->handle))) {
 			fprintf(stderr, "Not enough connection handles\n");
