@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2004-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -312,6 +312,10 @@ static char *opcode2str(uint16_t opcode)
 		return "sniff_subrate_req";
 	case 127 + (22 << 7):
 		return "sniff_subrate_res";
+	case 127 + (23 << 7):
+		return "pause_encryption_req";
+	case 127 + (24 << 7):
+		return "resume_encryption_req";
 	default:
 		return "unknown";
 	}
@@ -810,7 +814,7 @@ static inline void slot_offset_dump(int level, struct frame *frm)
 	uint16_t offset = LMP_U16(frm);
 	char addr[18];
 
-	ba2str((bdaddr_t *) frm->ptr, addr);
+	p_ba2str((bdaddr_t *) frm->ptr, addr);
 
 	p_indent(level, frm);
 	printf("slot offset %d\n", offset);
@@ -998,7 +1002,7 @@ static inline void sniff_subrate_dump(int level, struct frame *frm)
 	printf("max subrate %d\n", subrate);
 
 	p_indent(level, frm);
-	printf("min subrate zero timeout %d\n", timeout);
+	printf("min sniff timeout %d\n", timeout);
 
 	p_indent(level, frm);
 	printf("subrate instant 0x%4.4x\n", instant);
@@ -1165,6 +1169,8 @@ void lmp_dump(int level, struct frame *frm)
 	case 51:
 	case 56:
 	case 58:
+	case 127 + (23 << 7):
+	case 127 + (24 << 7):
 		return;
 	case 127 + (1 << 7):
 		accepted_ext_dump(level + 1, frm);
