@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2004-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -36,133 +36,6 @@
 
 #include "parser.h"
 
-#define TABLE_SIZE 20
-
-static struct {
-	uint16_t handle;
-	uint8_t dlci;
-	uint8_t opcode;
-	uint8_t status;
-	struct frame frm;
-} table[TABLE_SIZE];
-
-static void del_frame(uint16_t handle, uint8_t dlci)
-{
-	int i;
-
-	for (i = 0; i < TABLE_SIZE; i++)
-		if (table[i].handle == handle && table[i].dlci == dlci) {
-			table[i].handle = 0;
-			table[i].dlci   = 0;
-			table[i].opcode = 0;
-			table[i].status = 0;
-			if (table[i].frm.data)
-				free(table[i].frm.data);
-			memset(&table[i].frm, 0, sizeof(struct frame));
-			break;
-		}
-}
-
-static struct frame *add_frame(struct frame *frm)
-{
-	struct frame *fr;
-	void *data;
-	int i, pos = -1;
-
-	for (i = 0; i < TABLE_SIZE; i++) {
-		if (table[i].handle == frm->handle && table[i].dlci == frm->dlci) {
-			pos = i;
-			break;
-		}
-
-		if (pos < 0 && !table[i].handle && !table[i].dlci)
-			pos = i;
-	}
-
-	if (pos < 0)
-		return frm;
-
-	table[pos].handle = frm->handle;
-	table[pos].dlci   = frm->dlci;
-	fr = &table[pos].frm;
-
-	data = malloc(fr->len + frm->len);
-	if (!data) {
-		perror("Can't allocate OBEX stream buffer");
-		del_frame(frm->handle, frm->dlci);
-		return frm;
-	}
-
-	if (fr->len > 0)
-		memcpy(data, fr->ptr, fr->len);
-
-	if (frm->len > 0)
-		memcpy(data + fr->len, frm->ptr, frm->len);
-
-	if (fr->data)
-		free(fr->data);
-
-	fr->data     = data;
-	fr->data_len = fr->len + frm->len;
-	fr->len      = fr->data_len;
-	fr->ptr      = fr->data;
-	fr->dev_id   = frm->dev_id;
-	fr->in       = frm->in;
-	fr->ts       = frm->ts;
-	fr->handle   = frm->handle;
-	fr->cid      = frm->cid;
-	fr->num      = frm->num;
-	fr->dlci     = frm->dlci;
-	fr->channel  = frm->channel;
-	fr->audio_fd = frm->audio_fd;
-
-	return fr;
-}
-
-static uint8_t get_opcode(uint16_t handle, uint8_t dlci)
-{
-	int i;
-
-	for (i = 0; i < TABLE_SIZE; i++)
-		if (table[i].handle == handle && table[i].dlci == dlci)
-			return table[i].opcode;
-
-	return 0x00;
-}
-
-static void set_opcode(uint16_t handle, uint8_t dlci, uint8_t opcode)
-{
-	int i;
-
-	for (i = 0; i < TABLE_SIZE; i++)
-		if (table[i].handle == handle && table[i].dlci == dlci) {
-			table[i].opcode = opcode;
-			break;
-		}
-}
-
-static uint8_t get_status(uint16_t handle, uint8_t dlci)
-{
-	int i;
-
-	for (i = 0; i < TABLE_SIZE; i++)
-		if (table[i].handle == handle && table[i].dlci == dlci)
-			return table[i].status;
-
-	return 0x00;
-}
-
-static void set_status(uint16_t handle, uint8_t dlci, uint8_t status)
-{
-	int i;
-
-	for (i = 0; i < TABLE_SIZE; i++)
-		if (table[i].handle == handle && table[i].dlci == dlci) {
-			table[i].status = status;
-			break;
-		}
-}
-
 static char *opcode2str(uint8_t opcode)
 {
 	switch (opcode & 0x7f) {
@@ -396,6 +269,9 @@ void obex_dump(int level, struct frame *frm)
 
 		set_status(frm->handle, frm->dlci, status);
 
+		if (frm->len == 0)
+			break;
+
 		switch (opcode & 0x7f) {
 		case 0x00:	/* Connect */
 			version = get_u8(frm);
@@ -429,8 +305,3 @@ void obex_dump(int level, struct frame *frm)
 		parse_headers(level, frm);
 	}
 }
-
-void obex_clear(uint16_t handle, uint8_t dlci)
-{
-	del_frame(handle, dlci);
-}
