@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2000-2002  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2003-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2003-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -45,6 +45,7 @@ struct frame {
 	uint8_t		channel;
 	unsigned long	flags;
 	struct timeval	ts;
+	int		pppdump_fd;
 	int		audio_fd;
 };
 
@@ -74,9 +75,12 @@ struct frame {
 #define FILT_HIDP	0x0100
 #define FILT_HCRP	0x0200
 #define FILT_AVDTP	0x0400
+#define FILT_AVCTP	0x0800
 
 #define FILT_OBEX	0x00010000
 #define FILT_CAPI	0x00020000
+#define FILT_PPP	0x00040000
+#define FILT_ERICSSON	0x10000000
 #define FILT_CSR	0x1000000a
 #define FILT_DGA	0x1000000c
 
@@ -94,6 +98,7 @@ struct parser_t {
 	unsigned short defpsm;
 	unsigned short defcompid;
 	int state;
+	int pppdump_fd;
 	int audio_fd;
 };
 
@@ -101,7 +106,7 @@ extern struct parser_t parser;
 
 void init_parser(unsigned long flags, unsigned long filter, 
 		unsigned short defpsm, unsigned short defcompid,
-		int audio_fd);
+		int pppdump_fd, int audio_fd);
 
 static inline int p_filter(unsigned long f)
 {
@@ -121,7 +126,7 @@ static inline void p_indent(int level, struct frame *f)
 				struct tm tm;
 				time_t t = f->ts.tv_sec;
 				localtime_r(&t, &tm);
-				printf("%04d-%02d-%02d %02d:%02d:%02d.%06lu ", 
+				printf("%04d-%02d-%02d %02d:%02d:%02d.%06lu ",
 					tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday,
 					tm.tm_hour, tm.tm_min, tm.tm_sec, f->ts.tv_usec);
 			} else
@@ -136,6 +141,17 @@ static inline void p_indent(int level, struct frame *f)
 		printf("%*c", (level*2), ' ');
 }
 
+static inline void p_ba2str(const bdaddr_t *ba, char *str)
+{
+	if (parser.flags & DUMP_NOVENDOR) {
+		uint8_t b[6];
+
+		baswap((bdaddr_t *) b, ba);
+		sprintf(str, "%2.2X:%2.2X:%2.2X:*:*:*", b[0], b[1], b[2]);
+	} else
+		ba2str(ba, str);
+}
+
 /* get_uXX functions do byte swaping */
 
 static inline uint8_t get_u8(struct frame *frm)
@@ -184,8 +200,16 @@ char *get_uuid_name(int uuid);
 void set_proto(uint16_t handle, uint16_t psm, uint8_t channel, uint32_t proto);
 uint32_t get_proto(uint16_t handle, uint16_t psm, uint8_t channel);
 
+struct frame *add_frame(struct frame *frm);
+void del_frame(uint16_t handle, uint8_t dlci);
+
+uint8_t get_opcode(uint16_t handle, uint8_t dlci);
+void set_opcode(uint16_t handle, uint8_t dlci, uint8_t opcode);
+
+uint8_t get_status(uint16_t handle, uint8_t dlci);
+void set_status(uint16_t handle, uint8_t dlci, uint8_t status);
+
 void l2cap_clear(uint16_t handle);
-void obex_clear(uint16_t handle, uint8_t dlci);
 
 void ascii_dump(int level, struct frame *frm, int num);
 void hex_dump(int level, struct frame *frm, int num);
@@ -203,9 +227,14 @@ void cmtp_dump(int level, struct frame *frm);
 void hidp_dump(int level, struct frame *frm);
 void hcrp_dump(int level, struct frame *frm);
 void avdtp_dump(int level, struct frame *frm);
+void avctp_dump(int level, struct frame *frm);
 
 void obex_dump(int level, struct frame *frm);
 void capi_dump(int level, struct frame *frm);
+void ppp_dump(int level, struct frame *frm);
+void arp_dump(int level, struct frame *frm);
+void ip_dump(int level, struct frame *frm);
+void ericsson_dump(int level, struct frame *frm);
 void csr_dump(int level, struct frame *frm);
 void bpa_dump(int level, struct frame *frm);
 
