@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2001-2002  Ricky Yuen <ryuen@qualcomm.com>
- *  Copyright (C) 2003-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2003-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,7 @@
 
 #include <stdio.h>
 #include <errno.h>
+#include <ctype.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
@@ -154,6 +155,7 @@ static sdp_uuid_nam_lookup_table_t sdp_uuid_nam_lookup_table[] = {
 	{ SDP_UUID_SIM_ACCESS,               "SAP"          }, /* SAP */
 	{ SDP_UUID_PHONEBOOK_ACCESS_PCE,     "PBAP PCE"     }, /* PBAP */
 	{ SDP_UUID_PHONEBOOK_ACCESS_PSE,     "PBAP PSE"     }, /* PBAP */
+	{ SDP_UUID_PHONEBOOK_ACCESS,         "PBAP"         }, /* PBAP */
 	{ SDP_UUID_PNP_INFORMATION,          "PNPInfo"      },
 	{ SDP_UUID_GENERIC_NETWORKING,       "Networking"   },
 	{ SDP_UUID_GENERIC_FILE_TRANSFER,    "FileTrnsf"    },
@@ -167,6 +169,7 @@ static sdp_uuid_nam_lookup_table_t sdp_uuid_nam_lookup_table[] = {
 	{ SDP_UUID_VIDEO_SOURCE,             "VideoSource"  }, /* VDP */
 	{ SDP_UUID_VIDEO_SINK,               "VideoSink"    }, /* VDP */
 	{ SDP_UUID_VIDEO_DISTRIBUTION,       "VideoDist"    }, /* VDP */
+	{ SDP_UUID_APPLE_AGENT,              "AppleAgent"   },
 };
 
 #define SDP_UUID_NAM_LOOKUP_TABLE_SIZE \
@@ -206,7 +209,12 @@ static sdp_attr_id_nam_lookup_table_t sdp_attr_id_nam_lookup_table[] = {
 	{ SDP_ATTR_ID_NET_ACCESS_TYPE,                   "NetAccessType"      }, /* PAN */
 	{ SDP_ATTR_ID_MAX_NET_ACCESS_RATE,               "MaxNetAccessRate"   }, /* PAN */
 	{ SDP_ATTR_ID_IPV4_SUBNET,                       "IPv4Subnet"         }, /* PAN */
-	{ SDP_ATTR_ID_IPV6_SUBNET,                       "IPv6Subnet"         }  /* PAN */
+	{ SDP_ATTR_ID_IPV6_SUBNET,                       "IPv6Subnet"         }, /* PAN */
+	{ SDP_ATTR_ID_SUPPORTED_CAPABILITIES,            "SuppCapabilities"   }, /* Imaging */
+	{ SDP_ATTR_ID_SUPPORTED_FEATURES,                "SuppFeatures"       }, /* Imaging and Hansfree */
+	{ SDP_ATTR_ID_SUPPORTED_FUNCTIONS,               "SuppFunctions"      }, /* Imaging */
+	{ SDP_ATTR_ID_TOTAL_IMAGING_DATA_CAPACITY,       "SuppTotalCapacity"  }, /* Imaging */
+	{ SDP_ATTR_ID_SUPPORTED_REPOSITORIES,            "SuppRepositories"   }, /* PBAP */
 };
 
 #define SDP_ATTR_ID_NAM_LOOKUP_TABLE_SIZE \
@@ -295,10 +303,10 @@ static inline void print_int(uint8_t de_type, int level, int n, struct frame *fr
 		break;
 	case 16:/* 128-bit */
 		get_u128(frm, &val, &val2);
-		printf(" 0x%llx", val2);
+		printf(" 0x%jx", val2);
 		if (val < 0x1000000000000000LL)
 			printf("0");
-		printf("%llx", val);
+		printf("%jx", val);
 		return;
 	default: /* syntax error */
 		printf(" err");
@@ -307,7 +315,7 @@ static inline void print_int(uint8_t de_type, int level, int n, struct frame *fr
 		return;
 	}
 
-	printf(" 0x%llx", val);
+	printf(" 0x%jx", val);
 }
 
 static inline void print_uuid(int n, struct frame *frm, uint16_t *psm, uint8_t *channel)
@@ -359,16 +367,28 @@ static inline void print_uuid(int n, struct frame *frm, uint16_t *psm, uint8_t *
 
 static inline void print_string(int n, struct frame *frm, const char *name)
 {
-	char *s;
+	int i, hex = 0;
+
+	for (i = 0; i < n; i++) {
+		if (i == (n - 1) && ((char *) frm->ptr)[i] == '\0')
+			break;
+
+		if (!isprint(((char *) frm->ptr)[i])) {
+			hex = 1;
+			break;
+		}
+	}
 
 	printf(" %s", name);
-	if ((s = malloc(n + 1))) {
-		strncpy(s, frm->ptr, n);
-		s[n] = '\0';
-		printf(" \"%s\"", s);
-		free(s);
-	} else
-		perror("Can't allocate string buffer");
+	if (hex) {
+		for (i = 0; i < n; i++)
+			printf(" %02x", ((unsigned char *) frm->ptr)[i]);
+	} else {
+		printf(" \"");
+		for (i = 0; i < n; i++)
+			printf("%c", ((char *) frm->ptr)[i]);
+		printf("\"");
+	}
 
 	frm->ptr += n;
 	frm->len -= n;
@@ -587,7 +607,7 @@ static char *pid2str(uint8_t pid)
 
 static struct frame frame_table[FRAME_TABLE_SIZE];
 
-static int add_frame(struct frame *frm, int count)
+static int frame_add(struct frame *frm, int count)
 {
 	register struct frame *fr;
 	register unsigned char *data;
@@ -622,27 +642,28 @@ static int add_frame(struct frame *frm, int count)
 	if (fr->data)
 		free(fr->data);
 
-	fr->data     = data;
-	fr->data_len = len + count;
-	fr->len      = fr->data_len;
-	fr->ptr      = fr->data;
-	fr->dev_id   = frm->dev_id;
-	fr->in       = frm->in;
-	fr->ts       = frm->ts;
-	fr->handle   = frm->handle;
-	fr->cid      = frm->cid;
-	fr->num      = frm->num;
-	fr->channel  = frm->channel;
-	fr->audio_fd = frm->audio_fd;
+	fr->data       = data;
+	fr->data_len   = len + count;
+	fr->len        = fr->data_len;
+	fr->ptr        = fr->data;
+	fr->dev_id     = frm->dev_id;
+	fr->in         = frm->in;
+	fr->ts         = frm->ts;
+	fr->handle     = frm->handle;
+	fr->cid        = frm->cid;
+	fr->num        = frm->num;
+	fr->channel    = frm->channel;
+	fr->pppdump_fd = frm->pppdump_fd;
+	fr->audio_fd   = frm->audio_fd;
 
 	return pos;
 }
 
-static struct frame *get_frame(struct frame *frm, int count)
+static struct frame *frame_get(struct frame *frm, int count)
 {
 	register int pos;
 
-	pos = add_frame(frm, count);
+	pos = frame_add(frm, count);
 	if (pos < 0)
 		return frm;
 
@@ -740,9 +761,9 @@ void sdp_dump(int level, struct frame *frm)
 
 		if (cont == 0) {
 			/* Parse AttributeList */
-			print_attr_list(level + 1, get_frame(frm, count));
+			print_attr_list(level + 1, frame_get(frm, count));
 		} else
-			add_frame(frm, count);
+			frame_add(frm, count);
 
 		print_cont_state(level + 1, frm->ptr + count);
 		break;
@@ -773,9 +794,9 @@ void sdp_dump(int level, struct frame *frm)
 
 		if (cont == 0) {
 			/* Parse AttributeLists */
-			print_attr_lists(level + 1, get_frame(frm, count));
+			print_attr_lists(level + 1, frame_get(frm, count));
 		} else
-			add_frame(frm, count);
+			frame_add(frm, count);
 
 		print_cont_state(level + 1, frm->ptr + count);
 		break;
