@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2000-2002  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2003-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2003-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -75,6 +75,7 @@ enum {
 	WRITE,
 	RECEIVE,
 	SEND,
+	PPPDUMP,
 	AUDIO
 };
 
@@ -85,9 +86,11 @@ static int  defpsm = 0;
 static int  defcompid = DEFAULT_COMPID;
 static int  mode = PARSE;
 static int  permcheck = 1;
+static int  noappend = 0;
 static long flags;
 static long filter;
 static char *dump_file = NULL;
+static char *pppdump_file = NULL;
 static char *audio_file = NULL;
 static in_addr_t dump_addr = INADDR_LOOPBACK;
 static in_port_t dump_port = DEFAULT_PORT;
@@ -218,7 +221,8 @@ static void process_frames(int dev, int sock, int fd, unsigned long flags)
 		frm.data_len = len;
 		frm.dev_id = dev;
 		frm.in = 0;
-		frm.audio_fd = parser.audio_fd;
+		frm.pppdump_fd = parser.pppdump_fd;
+		frm.audio_fd   = parser.audio_fd;
 
 		cmsg = CMSG_FIRSTHDR(&msg);
 		while (cmsg) {
@@ -358,9 +362,9 @@ static int open_file(char *file, int mode, unsigned long flags)
 	struct btsnoop_hdr hdr;
 	int fd, len, open_flags;
 
-	if (mode == WRITE || mode == AUDIO) {
-		if (flags & DUMP_BTSNOOP)
-			open_flags = O_WRONLY | O_CREAT;
+	if (mode == WRITE || mode == PPPDUMP || mode == AUDIO) {
+		if (noappend || flags & DUMP_BTSNOOP)
+			open_flags = O_WRONLY | O_CREAT | O_TRUNC;
 		else
 			open_flags = O_WRONLY | O_CREAT | O_APPEND;
 	} else
@@ -603,8 +607,10 @@ static struct {
 	{ "hidp",	FILT_HIDP	},
 	{ "hcrp",	FILT_HCRP	},
 	{ "avdtp",	FILT_AVDTP	},
+	{ "avctp",	FILT_AVCTP	},
 	{ "obex",	FILT_OBEX	},
 	{ "capi",	FILT_CAPI	},
+	{ "ppp",	FILT_PPP	},
 	{ "csr",	FILT_CSR	},
 	{ "dga",	FILT_DGA	},
 	{ 0 }
@@ -644,10 +650,13 @@ static void usage(void)
 	"  -C, --cmtp=psm             PSM for CMTP\n"
 	"  -H, --hcrp=psm             PSM for HCRP\n"
 	"  -O, --obex=channel         Channel for OBEX\n"
+	"  -P, --ppp=channel          Channel for PPP\n"
+	"  -D, --pppdump=file         Extract PPP traffic\n"
 	"  -A, --audio=file           Extract SCO audio data\n"
 	"  -B, --btsnoop              Use BTSnoop file format\n"
 	"  -V, --verbose              Verbose decoding\n"
 	"  -Y, --novendor             No vendor commands or events\n"
+	"  -N, --noappend             No appending to existing files\n"
 	"  -h, --help                 Give this help list\n"
 	"      --usage                Give a short usage message\n"
 	);
@@ -670,11 +679,14 @@ static struct option main_options[] = {
 	{ "cmtp",		1, 0, 'C' },
 	{ "hcrp",		1, 0, 'H' },
 	{ "obex",		1, 0, 'O' },
+	{ "ppp",		1, 0, 'P' },
+	{ "pppdump",		1, 0, 'D' },
 	{ "audio",		1, 0, 'A' },
 	{ "btsnoop",		0, 0, 'B' },
 	{ "verbose",		0, 0, 'V' },
 	{ "novendor",		0, 0, 'Y' },
 	{ "nopermcheck",	0, 0, 'Z' },
+	{ "noappend",		0, 0, 'N' },
 	{ "help",		0, 0, 'h' },
 	{ 0 }
 };
@@ -683,11 +695,11 @@ int main(int argc, char *argv[])
 {
 	struct hostent *host;
 	struct in_addr addr;
-	int opt, fd = -1;
+	int opt, pppdump_fd = -1, audio_fd = -1;
 
 	printf("HCI sniffer - Bluetooth packet analyzer ver %s\n", VERSION);
 
-	while ((opt=getopt_long(argc, argv, "i:l:p:m:w:r:s:n:taxXRC:H:O:A:BVYZh", main_options, NULL)) != -1) {
+	while ((opt=getopt_long(argc, argv, "i:l:p:m:w:r:s:n:taxXRC:H:O:P:D:A:BVYZNh", main_options, NULL)) != -1) {
 		switch(opt) {
 		case 'i':
 			if (strcasecmp(optarg, "none") && strcasecmp(optarg, "system"))
@@ -776,6 +788,14 @@ int main(int argc, char *argv[])
 			set_proto(0, 0, atoi(optarg), SDP_UUID_OBEX);
 			break;
 
+		case 'P':
+			set_proto(0, 0, atoi(optarg), SDP_UUID_LAN_ACCESS_PPP);
+			break;
+
+		case 'D':
+			pppdump_file = strdup(optarg);
+			break;
+
 		case 'A':
 			audio_file = strdup(optarg);
 			break;
@@ -796,6 +816,10 @@ int main(int argc, char *argv[])
 			permcheck = 0;
 			break;
 
+		case 'N':
+			noappend = 1;
+			break;
+
 		case 'h':
 		default:
 			usage();
@@ -814,17 +838,20 @@ int main(int argc, char *argv[])
 	if (!filter)
 		filter = ~0L;
 
+	if (pppdump_file)
+		pppdump_fd = open_file(pppdump_file, PPPDUMP, flags);
+
 	if (audio_file)
-		fd = open_file(audio_file, AUDIO, flags);
+		audio_fd = open_file(audio_file, AUDIO, flags);
 
 	switch (mode) {
 	case PARSE:
-		init_parser(flags, filter, defpsm, defcompid, fd);
+		init_parser(flags, filter, defpsm, defcompid, pppdump_fd, audio_fd);
 		process_frames(device, open_socket(device, flags), -1, flags);
 		break;
 
 	case READ:
-		init_parser(flags, filter, defpsm, defcompid, fd);
+		init_parser(flags, filter, defpsm, defcompid, pppdump_fd, audio_fd);
 		read_dump(open_file(dump_file, mode, flags));
 		break;
 
@@ -834,7 +861,7 @@ int main(int argc, char *argv[])
 		break;
 
 	case RECEIVE:
-		init_parser(flags, filter, defpsm, defcompid, fd);
+		init_parser(flags, filter, defpsm, defcompid, pppdump_fd, audio_fd);
 		read_dump(wait_connection(dump_addr, dump_port));
 		break;
 
